@@ -159,6 +159,11 @@ def is_iscsi_device(dev, node_uuid):
     return iscsi_id in dev
 
 
+def is_nvme_device(dev):
+    """check whether the device path belongs to an NVMe drive. """
+    return "/dev/nvme" in dev
+
+
 def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
                     configdrive_mb, node_uuid, commit=True,
                     boot_option="netboot", boot_mode="bios",
@@ -198,6 +203,8 @@ def make_partitions(dev, root_mb, swap_mb, ephemeral_mb,
     # the device partitions as /dev/sda1 and not /dev/sda-part1.
     if is_iscsi_device(dev, node_uuid):
         part_template = dev + '-part%d'
+    elif is_nvme_device(dev):
+        part_template = dev + 'p%d'
     else:
         part_template = dev + '%d'
     part_dict = {}
@@ -283,7 +290,8 @@ def is_block_device(dev):
 
 def dd(src, dst):
     """Execute dd from src to dst."""
-    utils.dd(src, dst, 'bs=%s' % CONF.disk_utils.dd_block_size, 'oflag=direct')
+    utils.dd(src, dst, 'bs=%s' % CONF.disk_utils.dd_block_size, 'oflag=direct',
+             'conv=sparse')
 
 
 def qemu_img_info(path):
@@ -367,6 +375,9 @@ def destroy_disk_metadata(dev, node_uuid):
                               run_as_root=True,
                               use_standard_locale=True)
 
+    utils.execute('sgdisk', '-Z', dev, run_as_root=True,
+                  use_standard_locale=True)
+
     LOG.info("Disk metadata on %(dev)s successfully destroyed for node "
              "%(node)s", {'dev': dev, 'node': node_uuid})
 
@@ -578,9 +589,10 @@ def _is_disk_larger_than_max_size(device, node_uuid):
     :returns: True if total disk size exceeds 2TB. Returns False otherwise.
     """
     try:
-        disksize_bytes = utils.execute('blockdev', '--getsize64', device,
-                                       use_standard_locale=True,
-                                       run_as_root=True)
+        disksize_bytes, err = utils.execute('blockdev', '--getsize64',
+                                            device,
+                                            use_standard_locale=True,
+                                            run_as_root=True)
     except (processutils.UnknownArgumentError,
             processutils.ProcessExecutionError, OSError) as e:
         msg = (_('Failed to get size of disk %(disk)s for node %(node)s. '
@@ -589,7 +601,7 @@ def _is_disk_larger_than_max_size(device, node_uuid):
         LOG.error(msg)
         raise exception.InstanceDeployFailure(msg)
 
-    disksize_mb = int(disksize_bytes[0].strip()) // 1024 // 1024
+    disksize_mb = int(disksize_bytes.strip()) // 1024 // 1024
 
     return disksize_mb > MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR
 
@@ -786,6 +798,8 @@ def create_config_drive_partition(node_uuid, device, configdrive):
 
             if is_iscsi_device(device, node_uuid):
                 config_drive_part = '%s-part%s' % (device, new_part.pop())
+            elif is_nvme_device(device):
+                config_drive_part = '%sp%s' % (device, new_part.pop())
             else:
                 config_drive_part = '%s%s' % (device, new_part.pop())
 
